# Generated by Django 2.2.2 on 2019-07-06 22:02

import edc_sites.models
from django.db import migrations

import edc_metadata.managers


class Migration(migrations.Migration):
    dependencies = [("edc_metadata", "0013_auto_20190706_0706")]

    operations = [
        migrations.AlterModelManagers(
            name="crfmetadata",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_metadata.managers.CrfMetadataManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="requisitionmetadata",
            managers=[
                ("on_site", edc_sites.models.CurrentSiteManager()),
                ("objects", edc_metadata.managers.RequisitionMetadataManager()),
            ],
        ),
    ]
