# Generated by Django 4.2.3 on 2023-07-10 02:57

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_metadata", "0021_alter_crfmetadata_managers_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="crfmetadata",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("show_order",),
                "verbose_name": "Crf collection status",
                "verbose_name_plural": "Crf collection status",
            },
        ),
        migrations.AlterModelOptions(
            name="requisitionmetadata",
            options={
                "default_manager_name": "objects",
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "get_latest_by": "modified",
                "ordering": ("show_order",),
                "verbose_name": "Requisition collection status",
                "verbose_name_plural": "Requisition collection status",
            },
        ),
    ]
